<?php
/**
 * @file
 * This file contains functions for openlucius groups.
 */

/**
 * Function to get all content within a group.
 *
 * @param \stdClass $group_node
 *   The group node.
 */
function openlucius_core_delete_group($group_node) {

  // Initialize operations array for batch.
  $operations = array();

  // Get the users in this active group.
  $users_in_group = openlucius_core_fetch_users('group', $group_node->nid);

  // Check the users.
  if (!empty($users_in_group)) {

    // Chunk the users in pieces.
    $user_chunks = array_chunk($users_in_group, 1, TRUE);

    // Loop through the chunks.
    foreach ($user_chunks as $chunk) {

      // Each chunk has x user objects.
      foreach ($chunk as $user_object) {

        // Add new operation to the batch.
        $operations[] = array(
          'openlucius_core_remove_users_from_group_in_batch',
          array(
            $group_node,
            $user_object,
          ),
        );
      }
    }
  }

  // Get all node ids that have the shared_group_reference $group_node->nid.
  $node_ids = db_select('field_data_field_shared_group_reference', 'g')
    ->fields('g', array('entity_id'))
    ->condition('g.field_shared_group_reference_nid', $group_node->nid, '=')
    ->execute()->fetchCol();

  // Check the node ids.
  if (count($node_ids) > 0) {

    // Break up all of our data so each process does not time out.
    $node_chunks = array_chunk($node_ids, 20);

    // Add new operation to the batch.
    foreach ($node_chunks as $key => $nid) {

      // Add new operation to the batch.
      $operations[] = array(
        'openlucius_core_remove_nodes_from_group_in_batch',
        array(
          $group_node,
          $nid,
        ),
      );

      // Add new operation to the batch.
      $operations[] = array(
        'openlucius_core_delete_group_notifications',
        array(
          $group_node,
          $nid,
        ),
      );

      // Add new operation to the batch.
      $operations[] = array(
        'openlucius_core_delete_group_notifications_notification',
        array(
          $group_node,
          $nid,
        ),
      );

      // Add new operation to the batch.
      $operations[] = array(
        'openlucius_core_delete_group_node_references',
        array(
          $group_node,
          $nid,
        ),
      );
    }
  }

  // Put all that information into our batch array.
  $batch = array(
    'operations'    => $operations,
    'title'         => t('Delete group'),
    'init_message'  => t('Initializing'),
    'error_message' => t('An error occurred'),
    'finished'      => 'openlucius_core_delete_group_finished',
  );

  // Start processing the batch operations.
  batch_set($batch);
}

/**
 * @param \stdClass $group_node
 *   The group node object from openlucius_core_node_delete().
 * @param \stdClass $user_object
 *   The user object
 * @param array $context
 *   The $context array gathers batch context information about the execution
 *   (read), as well as 'return values' for the current operation (write)
 */
function openlucius_core_remove_users_from_group_in_batch($group_node, $user_object, &$context) {
  // Set context message for the user to see.
  $context['message'] = t('Removing @user from group @group.', array(
    '@user'  => $user_object->name,
    '@group' => $group_node->title,
  ));
  openlucius_core_remove_users_from_group($group_node->nid, $user_object->uid, TRUE);
}

/**
 * @param \stdClass $group_node
 *   The group node object from openlucius_core_node_delete().
 * @param int $nid
 *   The node id of the node to delete.
 * @param array $context
 *   The $context array gathers batch context information about the execution
 *   (read), as well as 'return values' for the current operation (write)
 */
function openlucius_core_remove_nodes_from_group_in_batch($group_node, $nid, &$context) {
  // Set context message for the user to see.
  $context['message'] = t('Removing nodes from group @group.', array(
    '@group' => $group_node->title,
  ));
  node_delete_multiple($nid);
}

/**
 * Function for removing group notifications.
 *
 * @param \stdClass $group_node
 *   The group node object from openlucius_core_node_delete().
 * @param int $nid
 *   The node id to remove the notifications for.
 * @param array $context
 *   The $context array gathers batch context information about the execution
 *   (read), as well as 'return values' for the current operation (write)
 */
function openlucius_core_delete_group_notifications($group_node, $nid, &$context) {
  // Set context message for the user to see.
  $context['message'] = t('Removing notifications from group @group.', array(
    '@group' => $group_node->title,
  ));
  // Remove the notifications where the nid is $nid.
  db_delete('openlucius_notifications')
    ->condition('id', $nid, 'IN')
    ->condition('type', 'node', '=')
    ->execute();
}

/**
 * Function for removing group heartbeat activity.
 *
 * @param \stdClass $group_node
 *   The group node object from openlucius_core_node_delete().
 * @param int $nid
 *   The node id to remove the notifications for.
 * @param array $context
 *   The $context array gathers batch context information about the execution
 *   (read), as well as 'return values' for the current operation (write)
 */
function openlucius_core_delete_group_heartbeat_activity($group_node, $nid, &$context) {
  // Set context message for the user to see.
  $context['message'] = t('Removing activity from group @group.', array(
    '@group' => $group_node->title,
  ));
  // Remove the activity where the nid is $nid.
  db_delete('heartbeat_activity')
    ->condition('nid', $nid, 'IN')
    ->condition('type', 'node', '=')
    ->execute();
}

/**
 * Function for removing the notifications notification.
 *
 * @param \stdClass $group_node
 *   The group node object from openlucius_core_node_delete().
 * @param int $nid
 *   The node id to remove the notifications for.
 * @param array $context
 *   The $context array gathers batch context information about the execution
 *   (read), as well as 'return values' for the current operation (write)
 */
function openlucius_core_delete_group_notifications_notification($group_node, $nid, &$context) {
  // Set context message for the user to see.
  $context['message'] = t('Removing notifications from group @group.', array(
    '@group' => $group_node->title,
  ));
  // Remove the activity where the nid is $nid.
  db_delete('openlucius_notifications_notification')
    ->condition('nid', $nid, 'IN')
    ->execute();
}

/**
 * Function for removing the node reference.
 *
 * @param \stdClass $group_node
 *   The group node object from openlucius_core_node_delete().
 * @param int $nid
 *   The node id to remove the reference for.
 * @param array $context
 *   The $context array gathers batch context information about the execution
 *   (read), as well as 'return values' for the current operation (write)
 */
function openlucius_core_delete_group_node_references($group_node, $nid, &$context) {
  // Set context message for the user to see.
  $context['message'] = t('Removing node references from group @group.', array(
    '@group' => $group_node->title,
  ));
  // Remove the activity where the rid is $nid.
  db_delete('openlucius_core_node_references')
    ->condition('rid', $nid, 'IN')
    ->execute();
}

/**
 * This function is called by the batch 'finished' parameter.
 *
 * @param bool $success
 *   Whether or not the batch was successfull.
 * @param array $results
 *   The array of results gathered so far by the batch processing,
 *   for the current operation to append its own.
 * @param array $operations
 *   An array that can be used to store persistent data between iterations.
 */
function openlucius_core_delete_group_finished($success, $results, $operations) {
  $message = $success ? t('Bulkdelete perform§ed successfully.') : t('Bulkdelete has not been finished successfully.');
  watchdog('bulkdelete', $message);
}
