<?php
/**
 * @file
 * This file contains helper functions for openlucius.
 */

/**
 * Function to obtain an icon based on type / path.
 *
 * @param string $type
 *   Either the node type or the path.
 *
 * @return string
 *   Returns a usable glyphicon, defaults to globe.
 */
function openlucius_core_icons($type) {

  $icons = array(
    'ol_group'             => 'record',
    'file'                 => 'file',
    'ol_message'           => 'envelope',
    'ol_text_document'     => 'font',
    'ol_todo_list'         => 'list-alt',
    'ol_todo'              => 'inbox',
    'ol_event'             => 'calendar',
    'group-files'          => 'file',
    'all-files'            => 'file',
    'group-messages'       => 'envelope',
    'all-messages'         => 'envelope',
    'group-textdocuments'  => 'font',
    'all-text-documents'   => 'font',
    'group-todo-lists'     => 'list-alt',
    'all-todo-lists'       => 'list-alt',
    'group-calendar/month' => 'calendar',
    'all-calendar'         => 'calendar',
    'trash'                => 'trash',
    'archived-groups'      => 'folder-close',
    'comment'              => 'comment',
    'group-users'          => 'user',
    'all-users'            => 'user',
    'companies-teams'      => 'user',
    'blocked-users'        => 'user',
    'user'                 => 'user',
    'user-calendar'        => 'inbox',
    'search'               => 'search',
    'node_add'             => 'plus-sign',
    'node_due_date'        => 'time',
    'node_label'           => 'flag',
    'node_time'            => 'time',
    'node_show_clients'    => 'eye-open',
    'node_hide_clients'    => 'eye-close',
  );

  // Allow other modules to add / edit icons.
  drupal_alter('openlucius_core_icons', $type);

  return isset($icons[$type]) ? $icons[$type] : 'globe';
}

/**
 * Function to fetch all user ids.
 * @return array
 *   Returns an array containing all of the active user id's.
 */
function openlucius_core_fetch_all_active_uids() {
  return db_select('users', 'u')
    ->fields('u', array('uid'))
    ->condition('status', 1, '=')
    ->execute()
    ->fetchCol();
}

/**
 * Function for quickly loading a user image in a given style.
 *
 * @param mixed $u
 *   Either a user id or a fully loaded user object.
 * @param string $style
 *   An image style to be applied to the image.
 *
 * @return string
 *   Returns the path to the styled image.
 * @throws \Exception
 */
function openlucius_core_fetch_user_image($u, $style = 'ol_50x50') {

  // If u is a number it's the uid otherwise it is a user object.
  $account = is_numeric($u) ? user_load($u) : $u;

  // The default 'avatar' image.
  $default_uri = drupal_get_path('profile', 'openlucius') . '/themes/openlucius/images/avatar.png';

  // Check if the user has a picture.
  if (!empty($account->picture)) {

    // Check if the file was loaded.
    if (!empty($account->picture->uri)) {

      if (file_exists($account->picture->uri)) {
        $image = theme('image_style', array(
          'style_name' => $style,
          'path'       => $account->picture->uri,
        ));
      }
      else {
        $image = theme('image', array(
          'style_name' => 'ol_50x50',
          'path'       => $default_uri,
        ));
      }
    }
    // Build path to default picture.
    else {
      $image = theme('image', array(
        'style_name' => 'ol_50x50',
        'path'       => $default_uri,
      ));
    }
  }

  // Build path to default picture.
  else {
    $image = theme('image', array(
      'style_name' => 'ol_50x50',
      'path'       => $default_uri,
    ));
  }

  return $image;
}

/**
 * Function to fetch the read nodes based on user id and an array of nodes.
 *
 * @param int $uid
 *   The user for whom the history tables should be read.
 * @param array $node_ids
 *   The node id's to be checked.
 * @param bool $data
 *   Whether the node-data should be returned or the count.
 *
 * @return mixed
 *   Returns either FALSE when not found or an array containing the node id's
 *   or the amount of items.
 */
function openlucius_core_fetch_read_nodes($uid, array $node_ids, $data = FALSE) {

  // Fetch read nodes based on node ids.
  $query = db_select('history', 'h')
    ->fields('h', array('nid'))
    ->condition('uid', $uid, '=')
    ->condition('nid', $node_ids, 'IN')
    ->execute();

  // Return row count.
  if (!$data) {
    return $query->rowCount();
  }

  // Return node id's.
  return $query->fetchCol();
}

/**
 * Function for fetching all published nodes id's of a certain type.
 *
 * @param string $type
 *   The node type to searched.
 *
 * @return mixed
 *   Returns either FALSE when not found or an array containing the node id's.
 */
function openlucius_core_fetch_nodes_of_type($type) {

  // All nodes of type.
  return db_select('node', 'n')
    ->fields('n', array('nid'))
    ->condition('n.type', check_plain($type), '=')
    ->condition('n.status', 1, '=')
    ->execute()
    ->fetchCol();
}

/**
 * Function to render a node field for the metadata block.
 *
 * @param \stdClass $node
 *   The node containing the data.
 *
 * @return string
 *   The rendered field.
 */
function openlucius_core_node_render_meta_fields(\stdClass $node) {

  $fields = array(
    'field_todo_list_reference'      => array(
      'icon' => openlucius_core_icons('ol_todo_list'),
      'tooltip' => t('The todo list this todo belongs to'),
    ),
    'field_todo_user_reference'      => array(
      'icon' => openlucius_core_icons('user'),
      'tooltip' => t('The assignee of this todo'),
    ),
    'field_todo_due_date_singledate' => array(
      'icon' => openlucius_core_icons('node_due_date'),
      'tooltip' => t('The due date of this todo'),
    ),
    'field_todo_label'               => array(
      'icon' => openlucius_core_icons('node_label'),
      'tooltip' => t('The current status of this todo'),
    ),
    'field_shared_show_clients'      => array(
      'icon' => openlucius_core_icons('node_show_clients'),
      'tooltip' => t('Can clients see this?'),
    ),
    'field_shared_time_maximum' => array(
      'icon' => openlucius_core_icons('node_due_date'),
      'tooltip' => t('The maximum amount of time'),
    ),
  );

  // Allow other modules to alter the fields / icons.
  drupal_alter('openlucius_core_node_meta_fields', $fields);

  // Initialize empty array for field renders.
  $list = array();

  // Load the wrapper.
  $node_wrapper = entity_metadata_wrapper('node', $node);

  // Loop through the fields.
  foreach ($fields as $field => $values) {

    // Skip if the user is a client.
    if ($field == 'field_shared_show_clients' && openlucius_core_user_is_client()) {
      continue;
    }
    // Skip if the node is a text document as well.
    if ($field == 'field_shared_show_clients' && $node->type == 'ol_text_document') {
      continue;
    }

    // Check if we have the field.
    if (isset($node->{$field})) {

      // For all field which are not a label.
      if ($field !== 'field_todo_label') {

        // Fetch field items.
        $items      = field_get_items('node', $node, $field, $node->language);

        if ($field == 'field_shared_show_clients') {
          $field_data = field_view_value('node', $node, $field, $items[0], array('type' => 'i18n_list_default'), $node->language);
          $render     = render($field_data);
        }
        else {
          $field_data = field_view_value('node', $node, $field, $items[0], array(), $node->language);
          $render     = render($field_data);
        }
      }

      // The todo label should link to the list with the filter.
      else {

        // Get the todo label value.
        $data   = $node_wrapper->{$field}->value();

        // Get the todo list node id.
        $todo_list_nid = $node->field_todo_list_reference[LANGUAGE_NONE][0]['nid'];

        // Create a link for the label to the list with filters.
        $render = l($data->name, 'node/' . $todo_list_nid,
          array(
            'query' => array(
              'field_todo_label_tid[]' => $data->tid,
              'field_todo_user_reference_uid' => 'All'
            ),
            'html' => TRUE
          )
        );
      }

      // Check the render.
      if (!empty($render)) {
        // Append items to list.
        $list[] = '<i data-toggle="tooltip" data-placement="left" title="' . $values['tooltip'] . '" class="glyphicon glyphicon-' . $values['icon'] . '"></i>&nbsp;' . $render;
      }
    }
  }

  return theme('item_list', array('items'      => $list,
                                  'attributes' => array('class' => array('openlucius_meta_info'))
  ));
}

/**
 * Function for loading a user by realname.
 *
 * @param string $name
 *   The realname for this user.
 *
 * @return bool|mixed
 *   Returns either the loaded user or FALSE when not found.
 */
function openlucius_core_realname_load_user_by_name($name) {
  $uid = db_select('realname', 'r')
    ->fields('r', array('uid'))
    ->condition('r.realname', $name, '=')
    ->execute()
    ->fetchField();

  // Check if the user was found.
  if (!empty($uid)) {
    return user_load($uid);
  }

  return FALSE;
}

/**
 * Function for loading a realname from a user id.
 * @param int $uid
 *   The numeric user id.
 */
function openlucius_core_get_realname_from_uid($uid) {
  return db_select('realname', 'r')
    ->fields('r', array('realname'))
    ->condition('r.uid', $uid, '=')
    ->execute()->fetchField();
}

/**
 * Function to get array of users with a specific role.
 *
 * @param int $rid
 *   The numeric role id.
 * @param bool $raw
 *   Whether or not to return only user ids.
 *
 * @return array
 *   Returns array of user ids or user objects.
 */
function openlucius_core_get_users_with_role($rid, $raw = TRUE) {

  // Select from user roles.
  $query = db_select('users_roles', 'r');

  // Select user id.
  $query->fields('r', array('uid'));

  // Add condition for role id.
  $query->condition('r.rid', $rid, '=');

  // Add condition for user status.
  $query->condition('u.status', 1, '=');

  // Join towards the users table.
  $query->join('users', 'u', 'r.uid = u.uid');

  // Get the result.
  $result = $query->execute()->fetchCol();

  // Return users as object or raw array.
  return $raw ? $result : user_load_multiple($result);
}

/**
 * Function to recursively reset numeric keys in an array.
 *
 * @param array $array
 *   The array to receive new keys.
 *
 * @return array
 *   Returns an array where the numeric keys have been replaced.
 */
function openlucius_core_replace_numeric_keys(array &$array) {
  $result = array();
  $count  = 0;
  foreach ($array as $key => $value) {
    if (is_int($key)) {
      $key = $count;
    }
    if (is_array($value)) {
      $value = openlucius_core_replace_numeric_keys($value);
    }
    $result[$key] = $value;
    $count++;
  }
  return $result;
}
